// PIC10F322 LED Candle
// Written by Y H Ng
#include <xc.h>

#define _XTAL_FREQ 31000

__CONFIG(FOSC_INTOSC & BOREN_OFF & WDTE_OFF & PWRTE_OFF & MCLRE_OFF & CP_OFF & LVP_OFF & LPBOR_OFF & BORV_LO & WRT_OFF);

void init() {

    CLKRCONbits.CLKROE = 0;     // Reference clock disabled.
    OSCCONbits.IRCF = 0b000;    // 31kHz internal oscillator.
    ANSELA = 0x00;              // all ports are digital for A/D.
    TRISA = 0b00000100;         // RA2 is for A/D pin.


}

void init_pwm() {

    PWM1CON = 0x00;
    PWM2CON = 0x00;
    PR2 = 63;
    T2CON = 0x00;
    T2CONbits.T2CKPS = 0b00;    // prescaler is 0.
    PWM1CONbits.PWM1EN = 1;
    PWM1CONbits.PWM1OE = 1;
    PWM2CONbits.PWM2EN = 0;
    PWM2CONbits.PWM2OE = 0;
    T2CONbits.TMR2ON = 1;
    PIR1bits.TMR2IF = 0;
    PIE1bits.TMR2IE = 0;
    INTCONbits.GIE = 0;

}

unsigned char init_adc() {

    unsigned char ADresult = 0;
    ADCONbits.ADCS = 0b000;     // Fosc/2.
    ADCONbits.CHS = 0b010;      // AN2 - RA2.
    ADCONbits.ADON = 1;
    ADCONbits.GO_nDONE = 1;
    while(ADCONbits.GO_nDONE);
    ADresult = ADRES;
    ADCONbits.ADON = 0;
    return ADresult;
}

void main() {

    unsigned short lfsr = 0x00AA; // Random value. As long as it is not 0, and also should be at least 1 at the LSB
    unsigned short lsb = 0;
    unsigned char temp1 = 0;
    unsigned long timeCount = 0;

    init();

    init_pwm();

    temp1 = init_adc();         // Load a random value into the LFSR

    lfsr = lfsr | (temp1 << 8);

    while (1) {

        lsb = lfsr & 0x0001;    // Reference: Axel Kamper - http://stackoverflow.com/questions/16891655/galois-lfsr-explanation-of-code
        lfsr >>= 1;
        if(lsb == 0x0001)
            lfsr ^= 0xB400;


        PWM1DCH = (lfsr & 0xff00) >> 8;
        PWM1DCL = lfsr & 0x00f0;
        while(!PIR1bits.TMR2IF); PIR1bits.TMR2IF = 0;

        timeCount++;

        if(timeCount == 108000) {           // Shut off candle right after approximately 3 hours.
            PWM1CON = 0x00;
            PWM2CON = 0x00;
            PORTA = 0x00;
            TRISA = 0x00;
            asm("sleep");
        }
        // 108000 * 100ms = 3 hours with extra code overhead (up to few ms).
        __delay_ms(100);

    }



}
